@@ -67,12 +67,16 @@ module VagrantPlugins
         Vagrant::Action::Builder.new.tap do |b|
           b.use ConfigValidate
           b.use Call, IsRunning do |env, b2|
-            # If the VM is running, then our work here is done, exit
-            next if env[:result]
+            # If the VM is running, run the necessary provisioners
+            if env[:result]
+              b2.use action_provision
+              next
+            end
 
             b2.use Call, IsSuspended do |env2, b3|
               # if vm is suspended resume it then exit
               if env2[:result]
+                b3.use CreateNetworks
                 b3.use ResumeDomain
                 next
               end
@@ -124,6 +128,7 @@ module VagrantPlugins
             end
 
             b2.use Call, IsSuspended do |env2, b3|
+              b3.use CreateNetworks if env2[:result]
               b3.use ResumeDomain if env2[:result]
             end
 
@@ -269,6 +274,7 @@ module VagrantPlugins
                 b3.use MessageNotSuspended
                 next
               end
+              b3.use CreateNetworks
               b3.use ResumeDomain
             end
           end
