@@ -34,6 +34,7 @@ module VagrantPlugins
           @uuid = config.uuid
           @cpus = config.cpus.to_i
           @cpu_features = config.cpu_features
+          @features = config.features
           @cpu_mode = config.cpu_mode
           @cpu_model = config.cpu_model
           @cpu_fallback = config.cpu_fallback
@@ -42,6 +43,7 @@ module VagrantPlugins
           @machine_type = config.machine_type
           @machine_arch = config.machine_arch
           @disk_bus = config.disk_bus
+          @disk_device = config.disk_device
           @nested = config.nested
           @memory_size = config.memory.to_i * 1024
           @management_network_mac = config.management_network_mac
@@ -61,6 +63,7 @@ module VagrantPlugins
                                "passwd='#{config.graphics_passwd}'"
                               end
           @video_type = config.video_type
+          @sound_type = config.sound_type
           @video_vram = config.video_vram
           @keymap = config.keymap
           @kvm_hidden = config.kvm_hidden
@@ -86,6 +89,9 @@ module VagrantPlugins
           # PCI device passthrough
           @pcis = config.pcis
 
+          # Watchdog device
+          @watchdog_dev = config.watchdog_dev
+
           # USB device passthrough
           @usbs = config.usbs
 
@@ -93,6 +99,9 @@ module VagrantPlugins
           @redirdevs = config.redirdevs
           @redirfilters = config.redirfilters
 
+          # smartcard device
+          @smartcard_dev = config.smartcard_dev
+
           # RNG device passthrough
           @rng = config.rng
 
@@ -169,6 +178,9 @@ module VagrantPlugins
           @cpu_features.each do |cpu_feature|
             env[:ui].info(" -- CPU Feature:       name=#{cpu_feature[:name]}, policy=#{cpu_feature[:policy]}")
           end
+          @features.each do |feature|
+            env[:ui].info(" -- Feature:           #{feature}")
+          end
           env[:ui].info(" -- Memory:            #{@memory_size / 1024}M")
           env[:ui].info(" -- Management MAC:    #{@management_network_mac}")
           env[:ui].info(" -- Loader:            #{@loader}")
@@ -186,6 +198,7 @@ module VagrantPlugins
           env[:ui].info(" -- Graphics Password: #{@graphics_passwd.empty? ? 'Not defined' : 'Defined'}")
           env[:ui].info(" -- Video Type:        #{@video_type}")
           env[:ui].info(" -- Video VRAM:        #{@video_vram}")
+          env[:ui].info(" -- Sound Type:	#{@sound_type}")
           env[:ui].info(" -- Keymap:            #{@keymap}")
           env[:ui].info(" -- TPM Path:          #{@tpm_path}")
 
@@ -230,6 +243,10 @@ module VagrantPlugins
             env[:ui].info(" -- RNG device model:  #{@rng[:model]}")
           end
 
+          if not @watchdog_dev.empty?
+            env[:ui].info(" -- Watchdog device:   model=#{@watchdog_dev[:model]}, action=#{@watchdog_dev[:action]}")
+          end
+
           @usbs.each do |usb|
             usb_dev = []
             usb_dev.push("bus=#{usb[:bus]}") if usb[:bus]
@@ -259,7 +276,20 @@ module VagrantPlugins
             end
           end
 
-          env[:ui].info(" -- Command line : #{@cmd_line}")
+          if not @smartcard_dev.empty?
+            env[:ui].info(" -- smartcard device:  mode=#{@smartcard_dev[:mode]}, type=#{@smartcard_dev[:type]}")
+          end
+
+          @qargs = config.qemu_args
+          if not @qargs.empty?
+            env[:ui].info(' -- Command line args: ')
+            @qargs.each do |arg|
+              msg = "    -> value=#{arg[:value]}, "
+              env[:ui].info(msg)
+            end
+          end
+
+          env[:ui].info(" -- Command line : #{@cmd_line}") unless @cmd_line.empty?
 
           # Create libvirt domain.
           # Is there a way to tell fog to create new domain with already
