@@ -17,7 +17,7 @@ module VagrantPlugins
           @logger = Log4r::Logger.new('vagrant_libvirt::action::create_network_interfaces')
           @management_network_name = env[:machine].provider_config.management_network_name
           config = env[:machine].provider_config
-          @nic_model_type = config.nic_model_type
+          @nic_model_type = config.nic_model_type || 'virtio'
           @nic_adapter_count = config.nic_adapter_count
           @app = app
         end
@@ -64,11 +64,17 @@ module VagrantPlugins
           end
 
           # Create each interface as new domain device.
+          @macs_per_network = Hash.new(0)
           adapters.each_with_index do |iface_configuration, slot_number|
             @iface_number = slot_number
             @network_name = iface_configuration[:network_name]
+            @source_options = {
+              network: @network_name
+            }
             @mac = iface_configuration.fetch(:mac, false)
             @model_type = iface_configuration.fetch(:model_type, @nic_model_type)
+            @driver_name = iface_configuration.fetch(:driver_name, false)
+            @driver_queues = iface_configuration.fetch(:driver_queues, false)
             @device_name = iface_configuration.fetch(:iface_name, false)
             template_name = 'interface'
             # Configuration for public interfaces which use the macvtap driver
@@ -77,6 +83,8 @@ module VagrantPlugins
               @mode = iface_configuration.fetch(:mode, 'bridge')
               @type = iface_configuration.fetch(:type, 'direct')
               @model_type = iface_configuration.fetch(:model_type, @nic_model_type)
+              @driver_name = iface_configuration.fetch(:driver_name, false)
+              @driver_queues = iface_configuration.fetch(:driver_queues, false)
               @portgroup = iface_configuration.fetch(:portgroup, nil)
               @network_name = iface_configuration.fetch(:network_name, @network_name)
               template_name = 'public_interface'
@@ -90,8 +98,10 @@ module VagrantPlugins
               raise Errors::TunnelPortNotDefined if @tunnel_port.nil?
               if @type == 'udp'
                 # default udp tunnel source to 127.0.0.1
-                @udp_tunnel_local_ip = iface_configuration.fetch(:tunnel_local_ip, '127.0.0.1')
-                @udp_tunnel_local_port = iface_configuration.fetch(:tunnel_local_port)
+                @udp_tunnel={
+                  address: iface_configuration.fetch(:tunnel_local_ip,'127.0.0.1'),
+                  port: iface_configuration.fetch(:tunnel_local_port)
+                }
               end
               # default mcast tunnel to 239.255.1.1. Web search says this
               # 239.255.x.x is a safe range to use for general use mcast
@@ -100,8 +110,13 @@ module VagrantPlugins
                            else
                              '127.0.0.1'
                            end
-              @tunnel_ip = iface_configuration.fetch(:tunnel_ip, default_ip)
-              @model_type = iface_configuration.fetch(:model_type, @nic_model_type)
+              @source_options = {
+                address: iface_configuration.fetch(:tunnel_ip, default_ip),
+                port: @tunnel_port
+              }
+              @tunnel_type = iface_configuration.fetch(:model_type, @nic_model_type)
+              @driver_name = iface_configuration.fetch(:driver_name, false)
+              @driver_queues = iface_configuration.fetch(:driver_queues, false)
               template_name = 'tunnel_interface'
               @logger.info("Setting up #{@type} tunnel interface using  #{@tunnel_ip} port #{@tunnel_port}")
             end
@@ -115,27 +130,46 @@ module VagrantPlugins
             @logger.info(message)
 
             begin
-              domain.attach_device(to_xml(template_name))
+              # FIXME: all options for network driver should be hash from Vagrantfile
+              driver_options = {}
+              driver_options[:name] = @driver_name if @driver_name
+              driver_options[:queues] = @driver_queues if @driver_queues
+              @udp_tunnel ||= {}
+              xml = if template_name == 'interface' or
+                       template_name == 'tunnel_interface'
+                      interface_xml(@type,
+                                    @source_options,
+                                    @mac,
+                                    @device_name,
+                                    @iface_number,
+                                    @model_type,
+                                    driver_options,
+                                    @udp_tunnel)
+                    else
+                      to_xml(template_name)
+                    end
+              domain.attach_device(xml)
             rescue => e
               raise Errors::AttachDeviceError,
                     error_message: e.message
             end
 
             # Re-read the network configuration and grab the MAC address
-            next if @mac
-            xml = Nokogiri::XML(domain.xml_desc)
             if iface_configuration[:iface_type] == :public_network
+              xml = Nokogiri::XML(domain.xml_desc)
+              source = "@network='#{@network_name}'"
               if @type == 'direct'
-                @mac = xml.xpath("/domain/devices/interface[source[@dev='#{@device}']]/mac/@address")
-              elsif !@portgroup.nil?
-                @mac = xml.xpath("/domain/devices/interface[source[@network='#{@network_name}']]/mac/@address")
-              else
-                @mac = xml.xpath("/domain/devices/interface[source[@bridge='#{@device}']]/mac/@address")
+                  source = "@dev='#{@device}'"
+              elsif @portgroup.nil?
+                source = "@bridge='#{@device}'"
               end
-            else
-              @mac = xml.xpath("/domain/devices/interface[source[@network='#{@network_name}']]/mac/@address")
+              if not @mac
+                macs = xml.xpath("/domain/devices/interface[source[#{source}]]/mac/@address")
+                @mac = macs[@macs_per_network[source]]
+                iface_configuration[:mac] = @mac.to_s
+              end
+              @macs_per_network[source] += 1
             end
-            iface_configuration[:mac] = @mac.to_s
           end
 
           # Continue the middleware chain.
@@ -150,7 +184,7 @@ module VagrantPlugins
               # Skip configuring the management network, which is on the first interface.
               # It's used for provisioning and it has to be available during provisioning,
               # ifdown command is not acceptable here.
-              next if slot_number == 0
+              next if slot_number.zero?
               next if options[:auto_config] === false
               @logger.debug "Configuring interface slot_number #{slot_number} options #{options}"
 
@@ -187,6 +221,39 @@ module VagrantPlugins
 
         private
 
+        def target_dev_name(device_name, type, iface_number)
+          if device_name
+            device_name
+          elsif type == 'netwrok'
+            "vnet#{iface_number}"
+          else
+            # TODO can we use same name vnet#ifnum?
+            #"tnet#{iface_number}" FIXME plugin vagrant-libvirt trying to create second tnet0 interface
+            "vnet#{iface_number}"
+          end
+        end
+
+        def interface_xml(type, source_options, mac, device_name,
+                          iface_number, model_type, driver_options,
+                          udp_tunnel={})
+          Nokogiri::XML::Builder.new do |xml|
+            xml.interface(type: type || 'network') do
+              xml.source(source_options) do
+                xml.local(udp_tunnel) if type == 'udp'
+              end
+              xml.mac(address: mac) if mac
+              xml.target(dev: target_dev_name(device_name, type, iface_number))
+              xml.alias(name: "net#{iface_number}")
+              xml.model(type: model_type.to_s)
+              xml.driver(driver_options)
+            end
+          end.to_xml(
+            save_with: Nokogiri::XML::Node::SaveOptions::NO_DECLARATION |
+                       Nokogiri::XML::Node::SaveOptions::NO_EMPTY_TAGS |
+                       Nokogiri::XML::Node::SaveOptions::FORMAT
+          )
+        end
+
         def find_empty(array, start = 0, stop = @nic_adapter_count)
           (start..stop).each do |i|
             return i unless array[i]
