@@ -54,10 +54,12 @@ module VagrantPlugins
               end
 
               # Iterface type
-              REXML::XPath.each(xml_descr, '/domain/devices/interface/model') do |iface_model|
-                if iface_model.attributes['type'] != config.nic_model_type
-                  descr_changed = true
-                  iface_model.attributes['type'] = config.nic_model_type
+              unless config.nic_model_type.nil?
+                REXML::XPath.each(xml_descr, '/domain/devices/interface/model') do |iface_model|
+                  if iface_model.attributes['type'] != config.nic_model_type
+                    descr_changed = true
+                    iface_model.attributes['type'] = config.nic_model_type
+                  end
                 end
               end
 
@@ -221,6 +223,12 @@ module VagrantPlugins
                 end
               end
 
+              # Sound device
+              if config.sound_type
+                sound = REXML::XPath.first(xml_descr,'/domain/devices/sound/model')
+              end
+
+
               # dtb
               if config.dtb
                 dtb = REXML::XPath.first(xml_descr, '/domain/os/dtb')
