@@ -64,6 +64,7 @@ module VagrantPlugins
       attr_accessor :cpu_model
       attr_accessor :cpu_fallback
       attr_accessor :cpu_features
+      attr_accessor :features
       attr_accessor :numa_nodes
       attr_accessor :loader
       attr_accessor :boot_order
@@ -71,6 +72,7 @@ module VagrantPlugins
       attr_accessor :machine_arch
       attr_accessor :machine_virtual_size
       attr_accessor :disk_bus
+      attr_accessor :disk_device
       attr_accessor :nic_model_type
       attr_accessor :nested
       attr_accessor :volume_cache
@@ -88,6 +90,7 @@ module VagrantPlugins
       attr_accessor :video_vram
       attr_accessor :keymap
       attr_accessor :kvm_hidden
+      attr_accessor :sound_type
 
       # Sets the information for connecting to a host TPM device
       # Only supports socket-based TPMs
@@ -116,6 +119,9 @@ module VagrantPlugins
       # Random number device passthrough
       attr_accessor :rng
 
+      # Watchdog device
+      attr_accessor :watchdog_dev
+
       # USB device passthrough
       attr_accessor :usbs
 
@@ -123,6 +129,9 @@ module VagrantPlugins
       attr_accessor :redirdevs
       attr_accessor :redirfilters
 
+      # smartcard device
+      attr_accessor :smartcard_dev
+
       # Suspend mode
       attr_accessor :suspend_mode
 
@@ -132,6 +141,9 @@ module VagrantPlugins
       # Attach mgmt network
       attr_accessor :mgmt_attach
 
+      # Additional qemuargs arguments
+      attr_accessor :qemu_args
+
       def initialize
         @uri               = UNSET_VALUE
         @driver            = UNSET_VALUE
@@ -157,12 +169,14 @@ module VagrantPlugins
         @cpu_model         = UNSET_VALUE
         @cpu_fallback      = UNSET_VALUE
         @cpu_features      = UNSET_VALUE
+        @features          = UNSET_VALUE
         @numa_nodes        = UNSET_VALUE
         @loader            = UNSET_VALUE
         @machine_type      = UNSET_VALUE
         @machine_arch      = UNSET_VALUE
         @machine_virtual_size = UNSET_VALUE
         @disk_bus          = UNSET_VALUE
+        @disk_device       = UNSET_VALUE
         @nic_model_type    = UNSET_VALUE
         @nested            = UNSET_VALUE
         @volume_cache      = UNSET_VALUE
@@ -178,6 +192,7 @@ module VagrantPlugins
         @graphics_passwd   = UNSET_VALUE
         @video_type        = UNSET_VALUE
         @video_vram        = UNSET_VALUE
+        @sound_type        = UNSET_VALUE
         @keymap            = UNSET_VALUE
         @kvm_hidden        = UNSET_VALUE
 
@@ -205,6 +220,9 @@ module VagrantPlugins
         # Random number device passthrough
         @rng = UNSET_VALUE
 
+        # Watchdog device
+        @watchdog_dev      = UNSET_VALUE
+
         # USB device passthrough
         @usbs              = UNSET_VALUE
 
@@ -212,6 +230,9 @@ module VagrantPlugins
         @redirdevs         = UNSET_VALUE
         @redirfilters      = UNSET_VALUE
 
+        # smartcard device
+        @smartcard_dev     = UNSET_VALUE
+
         # Suspend mode
         @suspend_mode      = UNSET_VALUE
 
@@ -220,6 +241,8 @@ module VagrantPlugins
 
         # Attach mgmt network
         @mgmt_attach       = UNSET_VALUE
+
+        @qemu_args  = []
       end
 
       def boot(device)
@@ -344,6 +367,20 @@ module VagrantPlugins
                    function:  options[:function])
       end
 
+      def watchdog(options = {})
+        if options[:model].nil?
+          raise 'Model must be specified.'
+        end
+
+        if @watchdog_dev == UNSET_VALUE
+            @watchdog_dev = {}
+        end
+
+        @watchdog_dev[:model] = options[:model]
+        @watchdog_dev[:action] = options[:action] || 'reset'
+      end
+
+
       def usb(options = {})
         if (options[:bus].nil? || options[:device].nil?) && options[:vendor].nil? && options[:product].nil?
           raise 'Bus and device and/or vendor and/or product must be specified. Check `lsusb` for these.'
@@ -378,6 +415,26 @@ module VagrantPlugins
                            allow: options[:allow])
       end
 
+      def smartcard(options = {})
+        if options[:mode].nil?
+          raise 'Option mode must be specified.'
+        elsif options[:mode] != 'passthrough'
+          raise 'Currently only passthrough mode is supported!'
+        elsif options[:type] == 'tcp' && (options[:source_mode].nil? || options[:source_host].nil? || options[:source_service].nil?)
+          raise 'If using type "tcp", option "source_mode", "source_host" and "source_service" must be specified.'
+        end
+
+        if @smartcard_dev == UNSET_VALUE
+          @smartcard_dev = {}
+        end
+
+        @smartcard_dev[:mode] = options[:mode]
+        @smartcard_dev[:type] = options[:type] || 'spicevmc'
+        @smartcard_dev[:source_mode] = options[:source_mode] if @smartcard_dev[:type] == 'tcp'
+        @smartcard_dev[:source_host] = options[:source_host] if @smartcard_dev[:type] == 'tcp'
+        @smartcard_dev[:source_service] = options[:source_service] if @smartcard_dev[:type] == 'tcp'
+      end
+
       # NOTE: this will run twice for each time it's needed- keep it idempotent
       def storage(storage_type, options = {})
         if storage_type == :file
@@ -430,12 +487,17 @@ module VagrantPlugins
           bus: options[:bus],
           cache: options[:cache] || 'default',
           allow_existing: options[:allow_existing],
-          shareable: options[:shareable]
+          shareable: options[:shareable],
+          serial: options[:serial]
         }
 
         @disks << disk # append
       end
 
+      def qemuargs(options = {})
+        @qemu_args << options if options[:value]
+      end
+
       # code to generate URI from a config moved out of the connect action
       def _generate_uri
         # builds the libvirt connection URI from the given driver config
@@ -515,13 +577,15 @@ module VagrantPlugins
           end
         @cpu_fallback = 'allow' if @cpu_fallback == UNSET_VALUE
         @cpu_features = [] if @cpu_features == UNSET_VALUE
+        @features = ['acpi','apic','pae'] if @features == UNSET_VALUE
         @numa_nodes = @numa_nodes == UNSET_VALUE ? nil : _generate_numa
         @loader = nil if @loader == UNSET_VALUE
         @machine_type = nil if @machine_type == UNSET_VALUE
         @machine_arch = nil if @machine_arch == UNSET_VALUE
         @machine_virtual_size = nil if @machine_virtual_size == UNSET_VALUE
         @disk_bus = 'virtio' if @disk_bus == UNSET_VALUE
-        @nic_model_type = 'virtio' if @nic_model_type == UNSET_VALUE
+        @disk_device = 'vda' if @disk_device == UNSET_VALUE
+        @nic_model_type = nil if @nic_model_type == UNSET_VALUE
         @nested = false if @nested == UNSET_VALUE
         @volume_cache = 'default' if @volume_cache == UNSET_VALUE
         @kernel = nil if @kernel == UNSET_VALUE
@@ -539,6 +603,7 @@ module VagrantPlugins
         @graphics_ip = '127.0.0.1' if @graphics_ip == UNSET_VALUE
         @video_type = 'cirrus' if @video_type == UNSET_VALUE
         @video_vram = 9216 if @video_vram == UNSET_VALUE
+        @sound_type = nil if @sound_type == UNSET_VALUE
         @keymap = 'en-us' if @keymap == UNSET_VALUE
         @kvm_hidden = false if @kvm_hidden == UNSET_VALUE
         @tpm_model = 'tpm-tis' if @tpm_model == UNSET_VALUE
@@ -574,6 +639,9 @@ module VagrantPlugins
         # Random number generator passthrough
         @rng = {} if @rng == UNSET_VALUE
 
+        # Watchdog device
+        @watchdog_dev = {} if @watchdog_dev == UNSET_VALUE
+
         # USB device passthrough
         @usbs = [] if @usbs == UNSET_VALUE
 
@@ -581,6 +649,9 @@ module VagrantPlugins
         @redirdevs = [] if @redirdevs == UNSET_VALUE
         @redirfilters = [] if @redirfilters == UNSET_VALUE
 
+        # smartcard device
+        @smartcard_dev = {} if @smartcard_dev == UNSET_VALUE
+
         # Suspend mode
         @suspend_mode = 'pause' if @suspend_mode == UNSET_VALUE
 
@@ -589,6 +660,8 @@ module VagrantPlugins
 
         # Attach mgmt network
         @mgmt_attach = true if @mgmt_attach == UNSET_VALUE
+
+        @qemu_args = [] if @qemu_args == UNSET_VALUE
       end
 
       def validate(machine)
@@ -614,6 +687,7 @@ module VagrantPlugins
           c = disks.dup
           c += other.disks
           result.disks = c
+
           c = cdroms.dup
           c += other.cdroms
           result.cdroms = c
